"""
Fundamental classification types for DRB.
"""

RB_CLASS = {
    -1: "Unclassified",
    0: "Unknown",
    1: "Real",
    2: "Bogus - Other",
    3: "Bogus - YinYang",
    4: "Bogus - Diffraction Spike",
    5: "Bogus - Bad Astrometry",
    6: "Bogus - Stripes",
    7: "Missing cutouts",
    8: "Bogus - Background",
    9: "Bogus - Failed imsub",
    10: "Bogus - Low S/N",
    11: "Bogus - Masking",
    12: "Bogus - Artifact",
    13: "Bogus - Ambiguous",
    14: "Real - Ambiguous",
}
